;;; -*- Syntax: Ansi-Common-Lisp; Base: 10; Mode: lisp; Package: imho -*-
;;; $Id: modal-editor.lisp,v 1.17 2003/03/26 19:52:16 adam Exp $
;;;
;;; Copyright (c) 1999 - 2003 onShore Development, Inc.
;;;
;;; See the file 'COPYING' in this directory for terms.
;;;
;;; A modal editor proxy-renders another class depending on its state.
;;; The editor can operate in a number of modes, each mode displaying
;;; a screen for editing various properties of an object.  Modes are
;;; selected with the mode-selector-popup class (see below).

(in-package :imho)

(defclass modal-editor (html-element)
  ((edit-mode
    :initform nil)
   (propagate
    :initform t
    :initarg :propagate)))

;; sets the first screen when the editor is initialized.
(defmethod default-mode ((self modal-editor))
  'modal-editor-empty)

;; init hook for modal-editor
(defmethod modal-init ((self t))
  )

;; cancel out of a mode, back to the default mode
(defun cancel-editing (class)
  (let ((editor (page-for-session class)))
    (setf (slot-value editor 'edit-mode) nil)))

;; put the editor into a mode
(defmethod start-editing ((self modal-editor) mode)
  (setf (slot-value self 'edit-mode) mode)
  (modal-init (get-modal-editor self)))

;; return a session-page for a mode
(defmethod get-modal-editor ((self modal-editor))
  (let ((mode (or (slot-value self 'edit-mode)
                  (default-mode self))))
    (with-slots (propagate)
        self
      (if propagate
          (page-for-session mode :value (element-value self))
          (page-for-session mode)))))

(defmethod preawake :before ((self modal-editor))
  (let ((editor (get-modal-editor self)))
    (setf (element-parent editor) self)
    (preawake editor)))
  
(defmethod awake :before ((self modal-editor))
  (let ((editor (get-modal-editor self)))
    (setf (element-parent editor) self)
    (awake editor)))

;; render the mode screen
(defmethod render-html ((self modal-editor) stream)
  (let ((editor (get-modal-editor self)))
    (setf (element-parent editor) self)
    (render-html editor stream)))

;; ------------------------------------------------------------
;; placeholder class to indicate that a modal editor has not got some
;; default contents defined

(defclass modal-editor-empty (html-element)
  ())

(defmethod render-html ((self modal-editor-empty) stream)
  (html-stream
   stream
   (:p
    (:princ "Empty modal editor"))))
