;;; -*- Syntax: Ansi-Common-Lisp; Base: 10; Mode: lisp; Package: imho -*-
;;; $Id: property-sheet.lisp,v 1.63 2003/03/26 19:52:16 adam Exp $
;;;
;;; Copyright (c) 1999 - 2003 onShore Development, Inc.
;;;
;;; See the file 'COPYING' in this directory for terms.

(in-package :imho)

(defun gradate-color (color)
  (flet ((extract-hex (offset)
           (parse-integer (subseq color offset (+ offset 2)) :radix 16))
         (insert-hex (rgb)
           (destructuring-bind (r g b)
               rgb
             (format nil "#~2x~2x~2x" r g b)))
         (scale-triple (pct r g b)
           (list (floor (* (/ pct 100.0) r))
                 (floor (* (/ pct 100.0) g))
                 (floor (* (/ pct 100.0) b)))))
    (let ((r (extract-hex 1))
          (g (extract-hex 3))
          (b (extract-hex 5)))
      (list color
            (insert-hex (scale-triple 80 r g b))
            (insert-hex (scale-triple 50 r g b))))))

(defun hairline-row (stream columns &optional (color +dark-gray+))
  (declare (optimize (speed 3)))
  (html-stream
   stream
   ((:tr :bgcolor color)
    ((:td :colspan columns)
     (write-spacer stream)))))

;; ------------------------------------------------------------
;; property-sheet
;;
;; for display of linked sets of editable properties

;; property-sheet-spec
;; (NAME CAPTION {:required :optional} TYPE EDITABLE-P)


(defclass property-sheet (html-form method-caller)
  ((title        :initarg :title
                 :initform nil)
   (color        :initarg :color
                 :initform "#ffffff")
   (border       :initarg :border
                 :initform 0)
   (header-bg    :initarg :header-bg
                 :initform +header-bg+)
   (top-hairline :initarg :top-hairline
                 :initform t)
   (result       :initarg :save-page
                 :initform nil)
   (property-set :initarg :property-set
                 :initform nil)
   (display-set  :initarg :display-set
                 :initform :standard)
   (control-set  :initarg :control-set
                 :initform '(:editing))
   (edit-labels  :initarg :labels
                 :initform t)
   (display-only :initarg :display-only
                 :initform nil)
   (edit-state   :initarg :edit-state
                 :initform nil))
  (:documentation
   "manages display and edit of a property set"))

(defun get-result-element-class (property-sheet)
  (car (slot-value property-sheet 'result)))

(defun get-result-element-frame (property-sheet)
  (or (cadr (slot-value property-sheet 'result))
      "_self"))

(defmethod render-html ((self property-sheet) stream)
  (with-slots (title color border header-bg top-hairline control-set property-set display-set edit-labels edit-state display-only)
    self
    (let* ((element-value  (element-value self))
           (current-fields (get-property-sheet-data element-value property-set display-set))
           (chopped-fields (break-list current-fields :br))
           (column-count-1 (length chopped-fields))
           (column-count (* 5 column-count-1)))
      (html-stream
       stream
       ((:table :width "100%" :cellpadding 0 :cellspacing 0 :border 0)
        (if top-hairline
            (hairline-row stream column-count))
        (when (or control-set title (and (not display-only)
                                         (not (eql edit-state t))))
          ;;             (hairline-row stream column-count)) used to be here!!
          (html-stream
           stream
           ((:tr :bgcolor header-bg)
            ((:td :align :left :valign :middle :colspan (1- column-count)
                  :class "list_header" :nowrap "nowrap"
                  :height "20")
             "&nbsp;&nbsp;&nbsp;&nbsp;"
             ((:font :color :black)
              (:princ (or title "&nbsp;"))))
            ((:td :align :right :valign :middle :class "list_header"
                  :nowrap "nowrap" :bgcolor header-bg)
             (unless (or display-only (eql edit-state t))
               (with-action (stream self edit-all)
                 (write-string "Edit All&nbsp;&nbsp;" stream)))))))
        ((:tr :bgcolor "#555555")
         (dotimes (x (length chopped-fields))
           (html-stream
            stream
            (:td
             (write-spacer stream :width 130))
            (:td
             (write-spacer stream :width 10))
            ((:td :width "100%")
             (write-spacer stream))
            (:td
             (write-spacer stream :width 10))
            (:td
             (write-spacer stream :width 25)))))
        
        (dotimes (y (apply #'max (mapcar #'length chopped-fields)))
          (html-stream
           stream
           (:tr
            (dotimes (x (length chopped-fields))
              (let ((property (nth y (nth x chopped-fields)))) 
                (destructuring-bind (property caption value type editable)
                    (or property '(nil "&nbsp;" nil nil nil))
                  (html-stream
                   stream
                   ((:td :nowrap "nowrap" :valign :top :align :right)
                    "&nbsp;&nbsp;&nbsp;&nbsp;"
                    (:princ (gettext caption))
                    "&nbsp;&nbsp;")
                   ((:td :valign :middle :bgcolor :white)
                    "&nbsp;")
                   ((:td :valign :middle :bgcolor :white)
                    (let ((display-type :display))
                      (when (and editable (not display-only) (editing? self property))
                        (setf display-type :edit))
                      (property-sheet-display type display-type self property value stream)))
                   ((:td :valign :middle :bgcolor :white)
                    "&nbsp;&nbsp;")
                   ((:td :width 1 :valign :bottom :align :right)
                    "&nbsp;"
                    (if (and editable edit-labels (not display-only) (not (editing? self property)))
                        (with-action (stream self start-editing (string property))
                          (write-string "Edit..." stream))
                        (write-string "&nbsp;" stream))
                    "&nbsp;"))))))
           (:tr
            ((:td :colspan column-count)
             (write-spacer stream))))))))))

(defmethod editing? ((self property-sheet) property)
  (with-slots (edit-state)
    self
    (or (eql edit-state t)
        (member property edit-state))))

;; WMs

(define-wm start-editing ((self property-sheet) (field keyword))
  (set-element-state self field)
  nil)

(define-wm edit-all ((self property-sheet))
  (setf (slot-value self 'edit-state) t)
  nil)

(defmethod set-element-state ((self property-sheet) state)
  (with-slots (edit-state)
    self
    (case state
      (:all
       (setf edit-state t))
      (:none
       (setf edit-state nil))
      (t
       (when (eql edit-state t)
         (setf edit-state nil))
       (if (member state edit-state)
           (setf edit-state (remove state edit-state))
           (pushnew state edit-state)))))
  (values))
  
(defsetf element-state set-element-state)

(define-wm all-fields ((self property-sheet) (display-set keyword))
  (setf (slot-value self 'display-set) display-set)
  nil)

(defvar *property-controls* (make-hash-table :test #'equal))

(defmacro define-presentation-type (type (display-type view-type &optional (lookup-type :class))
                                    &rest definition)
  (assert (member lookup-type '(:class :immediate :parented :deferred)) (lookup-type)
          "Presentation lookup type must be one of :class, :immediate or :deferred.")
  `(setf (gethash (list ',type ,display-type ,view-type) *property-controls*)
    (list ,lookup-type ',definition)))

(defun get-control-for-type (type display-type view-type)
  (or (gethash (list type display-type view-type)
               *property-controls*)
      (error "No ~s function for ~s in view of type ~s." display-type type view-type)))

(defun property-sheet-display (type display-type parent property property-value stream
                               &optional initargs)
  (property-sheet-display-view :html type display-type parent property property-value stream initargs))

(defun property-sheet-display-view (view-type type display-type parent property property-value stream initargs)
  (flet ((process-initargs (args)
           (let ((args (eval-initargs args)))
             (do ((added-args initargs (cddr added-args)))
                 ((null added-args))
               (setf (getf args (car added-args))
                     (cadr added-args)))
             args)))
    (let ((element-spec (get-control-for-type type display-type view-type)))
      (destructuring-bind (control-type control-spec)
          element-spec
        (ecase control-type
          (:deferred
              (setf control-spec (funcall (car control-spec) property-value property)))
          (:class
           t)
          (:parented
           (apply (car control-spec) `(,parent ,property-value ,stream
                                       ,@(process-initargs (cdr control-spec))))
           (return-from property-sheet-display-view))
          (:immediate
           (apply (car control-spec) `(,property-value ,stream
                                       ,@(process-initargs (cdr control-spec))))
           (return-from property-sheet-display-view)))
        (repeat-element parent stream
                        :class (car control-spec)
                        :name property
                        :value property-value
                        :initargs (process-initargs (cdr control-spec)))))))

(defun eval-initargs (list &aux result)
  (do ((list list (cddr list)))
      ((null list)
       (nreverse result))
    (push (car list) result)
    (push (eval (cadr list)) result)))
    

#+clim-stuff
(defmethod edited-property-value ((self property-sheet) property)
  (if-bind (element (child-element self property :if-does-not-exist t))
      (values (element-value element) t)
      (values nil nil)))

#+clim-stuff
(defun property-sheet-edit (self element property property-value stream)
  ;; this is just a temporary implementation allowing us to avoid
  ;; using output/input recording streams but still pass around
  ;; the parent element.  normally views are immutable but
  ;; in the short term we assume they are mutable
  (accept self :stream stream :view element
          ;; property sheets provide the prompt/field lable
          :prompt nil
          ;; identifies the element
          :query-identifier property
          ;; prepopulate with the current value
          :default property-value :insert-default t))

#+clim-stuff
(defmethod property-sheet-display ((self t) element property value stream)
  (declare (ignore property))
    ;; this is just a temporary implementation allowing us to avoid
  ;; using output/input recording streams but still pass around
  ;; the parent element
  (present value (if self self
                     (imho::presentation-type-of value))
           :stream stream :view element
           :acceptably t))

#+clim-stuff
(defmethod property-sheet-display-null ((self t) element stream)
  (present nil self :stream stream :view element
           :acceptably nil))

