;;; -*- Syntax: Ansi-Common-Lisp; Base: 10; Mode: lisp; Package: imho -*-
;;; $Id: tabbed-dialog.lisp,v 1.26 2003/03/26 19:52:16 adam Exp $
;;;
;;; Copyright (c) 1999 - 2003 onShore Development, Inc.
;;;
;;; See the file 'COPYING' in this directory for terms.

(in-package :imho)

;; ------------------------------------------------------------
;; component: tabbed-dialog

(defclass tabbed-dialog (html-element)
  ((visible-tab :initform nil
                :accessor visible-tab))
  (:documentation "show grouped detail views of something"))

(defun ensure-tab (element tab)
  (or (child-element element tab :if-does-not-exist t)
      (setf (child-element element tab nil)
            (session-instance tab))))

(defmethod preawake ((self tabbed-dialog))
  (with-slots (visible-tab)
    self
    (let ((tabs (get-available-tabs self)))
      (when (or (null visible-tab)
                (not (member visible-tab (mapcar #'car tabs))))
        (setf visible-tab (caar tabs)))
      (let ((visible-tab-element (ensure-tab self visible-tab)))
        (setf (element-value visible-tab-element)
              (element-value self))
        (preawake visible-tab-element)))))

(defmethod awake ((self tabbed-dialog))
  (awake (ensure-tab self (visible-tab self))))

(defmethod render-html ((self tabbed-dialog) stream)
  (with-slots (visible-tab)
    self
    (let* ((tabs (get-available-tabs self))
          (caption (third (assoc visible-tab tabs))))
      (html-stream
       stream
       ((:table :border 0 :cellpadding 0 :cellspacing 0 :width "100%")
        (:tr
         ((:td :bgcolor "#555555" :valign :top)
          (write-spacer stream :height 1)))
        ((:tr :valign "top")
         (:td
          ((:table :cellpadding 0 :cellspacing 0 :bgcolor "#888888" :border 0 :width "100%")
           (:tbody
            (:tr
             ((:td :width 2)
              (write-spacer stream))
             (:td
              ((:img :width 1 :height 24 :alt "" :src (local-href "images/spacer.gif"))))
             (dolist (tab tabs)
               (if (equal visible-tab (car tab))
                   (html-stream
                    stream
                    ((:td :width 10 :height 24 :valign :top :align :center :bgcolor +selected-tab+ :class "top_bar")
                     ((:img :alt "" :src (local-href "images/tab-left.gif"))))
                    ((:td :align :center :bgcolor +selected-tab+ :class "top_bar")
                     ((:font :color "black")
                      (write-string (cadr tab) stream)))
                    ((:td :width 10 :valign :top :align :center :bgcolor +selected-tab+ :class "top_bar")
                     ((:img :border 0 :alt "" :src (local-href "images/tab-right.gif")))))
                   (html-stream
                    stream
                    ((:td :width 10 :valign :top :align :center :bgcolor "#aaaaaa" :class "top_bar")
                     ((:img :alt "" :src (local-href "images/tab-left.gif"))))
                    
                    ((:td :align :center :bgcolor "#aaaaaa" :class "top_bar")
                     ((:font :color "#ffffff")
                     (with-action (stream self choose-tab (symbol-name (car tab)))
                       (write-string (cadr tab) stream))))
                    ((:td :width 10 :valign :top :align :center :bgcolor "#aaaaaa" :class "top_bar")
                     ((:img :alt "" :src (local-href "images/tab-right.gif"))))
                    ((:td :width 1 :align :center :bgcolor "#888888" :class "top_bar")
                     ((:img :width 1 :height 24 :alt "" :src (local-href "images/spacer.gif")))))))
             ((:td :width "100%" :align :center :bgcolor "#888888" :class "top_bar")
              "&nbsp;"))))))
        (:tr
         ((:td :bgcolor "#555555" :valign :top)
          (write-spacer stream :height 1)))
        (unless (string-nil-or-empty-p caption)
          (html-stream
           stream
           ((:tr :valign "top")
            ((:td :class "flabel_1" :valign :top :bgcolor +selected-tab+)
             (write-spacer stream :width 10 :height 14)
             (write-string caption stream)))
           (:tr
            ((:td :bgcolor "#555555" :valign :top)
             (write-spacer stream :height 1)))))
        ((:tr :valign :top)
         (:td
          (render-html (child-element self visible-tab) stream))))))))

(define-wm choose-tab ((self tabbed-dialog) (tab string))
  (when-bind (new-value (caar (member tab (get-available-tabs self)
                                      :key (lambda (x) (symbol-name (car x)))
                                      :test #'string=)))
    (setf (visible-tab self) new-value))
  nil)
