;; -*- Syntax: Ansi-Common-Lisp; Base: 10; Mode: lisp; Package: imho -*-
;; $Id: lisp-doc.lisp,v 1.1 2002/09/04 17:21:17 jesse Exp $

;; Copyright (C) 1999, 2000 onShore Inc.
;; See the file 'COPYING' in this directory for terms.

(in-package :imho)

(defapplication lisp-doc
    :base-url "lisp-doc"
    :initial-method lisp-doc)

(defun doc-start ()
  (init/application 'lisp-doc :start))

(defun doc-stop ()
  (init/application 'lisp-doc :stop))


(define-wm lisp-doc ((caller t))
  (session-instance 'lispdoc-frame))


(defclass lispdoc-frame (html-frameset)
  ()
  (:default-initargs :title "Lisp Documentation"
    :cols "200,*"
    :frames '((:control doc-control :auto)
              (:body    doc-body))))

(defclass doc-control (html-form)
  ((context :initform nil)))

(defmethod render-html ((self doc-control) stream)
  (with-slots (context)
    self
    (typecase context
      (package
       (render-package self context stream))
      (t
       (render-package-selector self stream)))))

(defvar *visible-packages* nil)

(defun render-package-selector (element stream)
  (let ((package-names (sort (mapcar #'package-name (list-all-packages)) #'string<)))
    (html-stream
     stream
     (:p "Top of Documentation Tree")
     (:p "Select a package to view.")
     ((:table :border 1)
      (dolist (package package-names)
        (when (or (null *visible-packages*)
                  (member package *visible-packages* :test #'equal))
          (html-stream
           stream
           ((:tr)
            (:td
             (with-action (stream element view-package package)
               (write-string package stream)))))))))))

(define-wm view-symbol ((caller doc-control) (symbol symbol))
  (let ((body (session-instance 'doc-body)))
    (cmsg "Sym Viewing: ~s" symbol)
    body))

(define-wm view-package ((caller doc-control) (package-name string))
  (with-slots (context)
    caller
    (when-bind (package (find-package (ensure-keyword package-name)))
      (setf context package)))
  nil)

(defun render-package (element package stream)
  (html-stream
   stream
   (:p
    (:princ-safe package))
   (:p
    (with-action (stream element view-package-selector "")
      (write-string "Return to package selector" stream)))
   ((:font :size "-2")
    (do-symbols (sym package)
      (html-stream
       stream
       (with-action (stream element (view-symbol "BODY") sym)
         (html-stream
          stream
          (:princ-safe sym)))
       :br)))))

(define-wm view-package-selector ((self doc-control))
  (setf (slot-value self 'context) nil)
  nil)

(define-wm doc-control ((caller t))
  (session-instance 'doc-control))
  
(defclass doc-body (html-form)
  ())

(defmethod render-html ((self doc-body) stream)
  (write-string  "Body" stream))

(define-wm doc-body ((caller t))
  (session-instance 'doc-body))
  
  



